# PopGen 1.1 is A Synthetic Population Generator for Advanced
# Microsimulation Models of Travel Demand
# Copyright (C) 2009, Arizona State University
# See PopGen/License

from PyQt4.QtCore import *
from PyQt4.QtGui import *
from PyQt4.QtSql import *

import shutil, urllib, os

from database.createDBConnection import createDBC
from summary_page import SummaryPage
from data_menu.data_process_status import DataDialog
from data_menu.display_data import DisplayTable
from data_menu.sf_data import AutoImportSF2000Data
from file_menu.open_project import SaveFile
from misc.widgets import RecodeDialog, VariableSelectionDialog, CreateVariable, NameDialog, DeleteRows

from misc.errors import *

from default_census_cat_transforms import *

class QTreeWidgetCMenu(QTreeWidget):
    def __init__(self, project=None, parent = None):
        super(QTreeWidgetCMenu, self).__init__(parent)
        self.setMinimumSize(350, 400)
        self.setColumnCount(2)
        self.setHeaderLabels(["Name", "Value"])
        self.setColumnWidth(0, 150)
        self.setItemsExpandable(True)
        self.setEnabled(False)
        self.project = project
        self.tables = []
        self.item = None
        


    def contextMenuEvent(self, event):
        menu = QMenu()
        importDataAction = menu.addAction("&Import Data")
        editProjectAction = menu.addAction("&Edit Project")

        menuTableEdit = QMenu()
        displayTableAction = menuTableEdit.addAction("Display")
        menuTableEdit.addSeparator()
        createVarAction = menuTableEdit.addAction("Create New Variable")
        recodeCatsAction = menuTableEdit.addAction("Recode Categories")
        deleteColAction = menuTableEdit.addAction("Delete Column(s)")
        deleteRowsAction = menuTableEdit.addAction("Delete Record(s)")
        dropAction = menuTableEdit.addAction("Delete Table")
        copyAction = menuTableEdit.addAction("Copy Table")
        renameAction = menuTableEdit.addAction("Rename Table")
        menuTableEdit.addSeparator()
        exportActionCSV = menuTableEdit.addAction("Export Table (CSV format)")
        exportActionTab = menuTableEdit.addAction("Export Table (Tab-delimited format)")
        menuTableEdit.addSeparator()
        defaultTransforAction = menuTableEdit.addAction("Default Transformation")

        self.connect(importDataAction, SIGNAL("triggered()"), self.importData)
        self.connect(editProjectAction, SIGNAL("triggered()"), self.editProject)
        self.connect(displayTableAction, SIGNAL("triggered()"), self.displayTable)
        self.connect(recodeCatsAction, SIGNAL("triggered()"), self.modifyCategories)
        self.connect(createVarAction, SIGNAL("triggered()"), self.createVariable)
        self.connect(deleteColAction, SIGNAL("triggered()"), self.deleteColumns)
        self.connect(deleteRowsAction, SIGNAL("triggered()"), self.deleteRows)
        self.connect(copyAction, SIGNAL("triggered()"), self.copyTable)
        self.connect(renameAction, SIGNAL("triggered()"), self.renameTable)
        self.connect(dropAction, SIGNAL("triggered()"), self.dropTable)

        self.connect(exportActionCSV, SIGNAL("triggered()"), self.exportTableCSV)
        self.connect(exportActionTab, SIGNAL("triggered()"), self.exportTableTab)
        self.connect(defaultTransforAction, SIGNAL("triggered()"), self.defaultTransformations)

        
        if self.item is not None:
            if self.item.parent() is None:
                menu.exec_(QCursor.pos())
                self.item=None

            else:
                if self.item.parent().text(0) == 'Project Tables' or self.item.parent().text(0) == 'Scenario Tables':
                    self.tablename = self.item.text(0)
                    self.parentText = self.item.parent().text(0)
                    menuTableEdit.exec_(QCursor.pos())
                    self.item=None



    def exportTableCSV(self):
        #tablename = self.item.text(0)
        fileDlg = SaveFile(self.project, "csv", self.tablename, self.parentText)

    def exportTableTab(self):
        #tablename = self.item.text(0)
        fileDlg = SaveFile(self.project, "dat", self.tablename, self.parentText)


    def deleteRows(self):
        #tablename = self.item.text(0)

        #parentText = self.item.parent().text(0)
        if self.parentText == 'Project Tables':
            database = self.project.name
        elif self.parentText == 'Scenario Tables':
            database = '%s%s%s' %(self.project.name, 'scenario', self.project.scenario)            
            
        self.populateVariableDictionary(self.tablename)

        projectDBC = createDBC(self.project.db, database)
        projectDBC.dbc.open()

        deleteRows = DeleteRows(self.project, self.parentText, self.tablename, self.variableTypeDictionary, "Delete Records", "modifydata")
        if deleteRows.exec_():
            projectDBC.dbc.open()
            reply = QMessageBox.question(self, "Delete Records", "Would you like to continue?",
                                         QMessageBox.Yes| QMessageBox.Cancel)
            if reply == QMessageBox.Yes:

                query = QSqlQuery(projectDBC.dbc)
                whereExpression = deleteRows.whereEdit.toPlainText()
                #print whereExpression, 'is the text'
                if not whereExpression == "":

                    if not query.exec_("""delete from %s where %s""" %(self.tablename, whereExpression)):
                        raise FileError, query.lastError().text()
                else:
                    QMessageBox.warning(self, "Delete Records", """No filter expression selected, 0 records deleted.""",
                                        QMessageBox.Ok)

        projectDBC.dbc.close()


    def copyTable(self):
        #tablename = self.item.text(0)

        parent = self.item.parent().text(0)
        if self.parentText == 'Project Tables':
            database = self.project.name
        elif self.parentText == 'Scenario Tables':
            database = '%s%s%s' %(self.project.name, 'scenario', self.project.scenario)            

        copyNameDialog = NameDialog("Copy Table - %s" %self.tablename)
        if copyNameDialog.exec_():
            newTablename = copyNameDialog.nameLineEdit.text()
            projectDBC = createDBC(self.project.db, database)
            projectDBC.dbc.open()

            query = QSqlQuery(projectDBC.dbc)
            if not query.exec_("""create table %s select * from %s""" %(newTablename, self.tablename)):
                raise FileError, query.lastError().text()
            self.populate()
            projectDBC.dbc.close()

    def renameTable(self):
        #tablename = self.item.text(0)
        parent = self.item.parent().text(0)
        if self.parentText == 'Project Tables':
            database = self.project.name
        elif self.parentText == 'Scenario Tables':
            database = '%s%s%s' %(self.project.name, 'scenario', self.project.scenario)            

        renameNameDialog = NameDialog("Rename Table - %s" %self.tablename)
        if renameNameDialog.exec_():
            newTablename = renameNameDialog.nameLineEdit.text()

            projectDBC = createDBC(self.project.db, database)
            projectDBC.dbc.open()

            query = QSqlQuery(projectDBC.dbc)
            if not query.exec_("""alter table %s rename to %s""" %(self.tablename, newTablename)):
                raise FileError, query.lastError().text()
            self.populate()
            projectDBC.dbc.close()

    def dropTable(self):
        #tablename = self.item.text(0)

        parent = self.item.parent().text(0)
        if self.parentText == 'Project Tables':
            database = self.project.name
        elif self.parentText == 'Scenario Tables':
            database = '%s%s%s' %(self.project.name, 'scenario', self.project.scenario)            

        reply = QMessageBox.question(self, "Delete Table - %s" %self.tablename, "Do you wish to continue?",
                                     QMessageBox.Yes| QMessageBox.Cancel)
        if reply == QMessageBox.Yes:
            projectDBC = createDBC(self.project.db, database)

            projectDBC.dbc.open()

            query = QSqlQuery(projectDBC.dbc)
            if not query.exec_("""drop table %s""" %self.tablename):
                raise FileError, query.lastError().text()
            self.populate()
            projectDBC.dbc.close()


    def click(self, item):
        self.item = item

    def createVariable(self):
        #parent = self.item.parent().text(0)
        if self.parentText == 'Project Tables':
            database = self.project.name
        elif self.parentText == 'Scenario Tables':
            database = '%s%s%s' %(self.project.name, 'scenario', self.project.scenario)            

        projectDBC = createDBC(self.project.db, database)
        projectDBC.dbc.open()

        #tablename = self.item.text(0)
        self.populateVariableDictionary(self.tablename)

        create = CreateVariable(self.project, database, self.tablename, self.variableTypeDictionary, "Create New Variable", "modifydata")
        if create.exec_():
            projectDBC.dbc.open()
            newVarName = create.newVarNameEdit.text()
            numericExpression = create.formulaEdit.toPlainText()
            whereExpression = create.whereEdit.toPlainText()
            if len(whereExpression)<1:
                whereExpression = '1'
            if len(numericExpression) <1:
                QMessageBox.warning(self, "Data", QString("""Invalid numeric expression, enter again"""))
            else:
                query = QSqlQuery(projectDBC.dbc)
                if not query.exec_("""alter table %s add column %s bigint""" %(self.tablename, newVarName)):
                    print ("""alter table %s add column %s bigint""" %(self.tablename, newVarName))
                    raise FileError, query.lastError().text()
                if not query.exec_("""update %s set %s = %s where %s""" %(self.tablename, newVarName,
                                                                          numericExpression, whereExpression)):
                    raise FileError, query.lastError().text()

        projectDBC.dbc.close()



    def displayTable(self):
        #tablename = self.item.text(0)

        disp = DisplayTable(self.project, "%s" %self.tablename, self.parentText)
        disp.exec_()

    def modifyCategories(self):
        #parentText = self.item.parent().text(0)
        """
        if parentText == 'Project Tables':
            database = self.project.name
        elif parentText == 'Scenario Tables':
            database = '%s%s%s' %(self.project.name, 'scenario', self.project.scenario)            
        projectDBC = createDBC(self.project.db, database)

        projectDBC.dbc.open()
        """
        #tablename = self.item.text(0)
        modify = RecodeDialog(self.project, self.parentText, self.tablename, title = "Recode Categories - %s" %self.tablename, icon = "modifydata")
        modify.exec_()

        #projectDBC.dbc.close()


    def deleteColumns(self):
        #parent = self.item.parent().text(0)

        if self.parentText == 'Project Tables':
            database = self.project.name
        elif self.parentText == 'Scenario Tables':
            database = '%s%s%s' %(self.project.name, 'scenario', self.project.scenario)            

        projectDBC = createDBC(self.project.db, database)

        #tablename = self.item.text(0)
        self.populateVariableDictionary(self.tablename)
        projectDBC.dbc.open()
        query = QSqlQuery(projectDBC.dbc)

        title = "Delete Dialog - %s" %self.tablename
        deleteVariablesdia = VariableSelectionDialog(self.variableTypeDictionary, title = title, icon = "modifydata",
                                                     warning = "Note: Select variables to delete.")

        if deleteVariablesdia.exec_():
            deleteVariablesSelected = deleteVariablesdia.selectedVariableListWidget.variables

            for i in deleteVariablesSelected:
                if not query.exec_("""alter table %s drop %s""" %(self.tablename, i)):
                    raise FileError, query.lastError().text()

        projectDBC.dbc.close()


    def defaultTransformations(self):
        projectDBC = createDBC(self.project.db, self.project.name)
        projectDBC.dbc.open()

        query = QSqlQuery(projectDBC.dbc)

        #tablename = self.item.text(0)

        checkPUMSTableTransforms = False
        checkSFTableTransforms = False

        if not self.project.sampleUserProv.userProv:
            if self.tablename == 'housing_pums':
                if self.project.sampleUserProv.defSource == 'Census 2000':
                    queries = DEFAULT_HOUSING_PUMS2000_QUERIES
                    checkPUMSTableTransforms = True
                else:
                    queries = DEFAULT_HOUSING_PUMSACS_QUERIES
                    checkPUMSTableTransforms = True

            if self.tablename == 'person_pums':
                if self.project.sampleUserProv.defSource == 'Census 2000':
                    queries = DEFAULT_PERSON_PUMS2000_QUERIES
                    checkPUMSTableTransforms = True
                else:
                    queries = DEFAULT_PERSON_PUMSACS_QUERIES
                    checkPUMSTableTransforms = True

            if checkPUMSTableTransforms:
                for i in queries:
                    print "Executing Query: %s" %i
                    if not query.exec_("""%s""" %i):
                        if not query.lastError().number() == 1051:
                            print "FileError: %s" %query.lastError().text()


        if not self.project.controlUserProv.userProv:
            if self.tablename[:13] == 'mastersftable':
                checkSFTableTransforms = True

            if checkSFTableTransforms:
                if self.project.controlUserProv.defSource == 'Census 2000':
                    queries = DEFAULT_SF2000_QUERIES
                else:
                    queries = DEFAULT_SFACS_QUERIES

                    # Edit by Dae on 10/30/2013
                    # Because ACS data at the Tract level include GQ,
                    # it is not necessary the below calculation to obtain GQ.
		    if self.project.resolution == 'County' or self.project.resolution == 'Tract':
                        queries.insert(142, "update %s set groupquarter1 = B26001000001")
                        queries.insert(143, "update %s set groupquarter2 = 0")
		    else:
                        queries.insert(142, "update %s set groupquarter1 = persontot - persontoteq")
                        queries.insert(143, "update %s set groupquarter1 = 0 where groupquarter1 <= 0")


                for i in queries:
                    print "Executing Query: %s" %i
                    if not query.exec_(i %self.tablename):
                        print "FileError: %s" %query.lastError().text()

                # Added by Dae on 11/05/2013
                if self.project.controlUserProv.defSource <> 'Census 2000':
                    self.gq_ratio_from_sample(query)

        if not (checkPUMSTableTransforms or checkSFTableTransforms):
            QMessageBox.warning(self, "Default Transformations", 
                                """No default transformations are defined for the table""",
                                QMessageBox.Ok)
            #print "FileError: The file does not have default transformations"

        projectDBC.dbc.close()
        self.populate()


    # Add by Dae on 10/25/2013 to calculate noninstitutionalized group quarter by ratios from PUMS sample.        
    def gq_ratio_from_sample(self, query):

        sqlstatement = ""
        if self.project.sampleUserProv.defSource == 'ACS 2007-2011' or self.project.sampleUserProv.defSource == 'ACS 2008-2012':
            sqlstatement = """select puma, relp, sum(pwgtp) from person_raw where relp >= 13 group by puma, relp order by puma, relp"""
        else:
            sqlstatement = """select puma, rel, sum(pwgtp) from person_raw where rel >= 13 group by puma, rel order by puma, rel"""

	#print "Executing Query: %s" %sqlstatement
        if not query.exec_(sqlstatement):
            raise FileError, self.query.lastError().text()
        else:
            gqratios = {}
            odd = 0
            numb13 = 0
            numb14 = 0
            while (query.next()):
                puma = query.value(0)
                relp = query.value(1)

                if relp == 13:
                    numb13 = query.value(2)
                else:
                    numb14 = query.value(2)
                    
                odd += 1
                if odd == 2 and relp == 14:
                    gqratios['%s'%puma.toString()] = float(numb14.toString()) / (float(numb13.toString()) + float(numb14.toString()))
                    odd = 0

        #print gqratios
        state = self.project.stateCode[self.project.state]
        pumas = gqratios.keys()
        queries1 = []
        for puma in pumas:
	    tmpquery = ""
	    if self.project.resolution == 'County':
		tmpquery = """select county from geocorr where state = %s and pumano = %s group by county""" %(state, puma)
	    else:
		tmpquery = """select county, tract from geocorr where state = %s and pumano = %s group by county, tract""" %(state, puma)

            if not query.exec_(tmpquery):
                raise FileError, self.query.lastError().text()
            else:
                ratio = gqratios[puma]
                while (query.next()):
                    county = query.value(0).toString()
		    tract = ""
		    if self.project.resolution <> 'County':
                        tract = " and tract = %s" %(query.value(1).toString())

                    temp1 = "update gq_marginals set groupquarter2 = groupquarter1*%s where county = %s%s" %(ratio, county, tract)
                    queries1.append(temp1)
                    temp1 = "update gq_marginals set groupquarter1 = groupquarter1*%s where county = %s%s" %((1-ratio), county, tract)
                    queries1.append(temp1)
                    

        for i in queries1:
	    #print "Executing Query: %s" %i
            if not query.exec_(i):
                print "FileError: %s" %query.lastError().text()


    def populateVariableDictionary(self, tablename):
        #parent = self.item.parent().text(0)

        if self.parentText == 'Project Tables':
            database = self.project.name
        elif self.parentText == 'Scenario Tables':
            database = '%s%s%s' %(self.project.name, 'scenario', self.project.scenario)            


        projectDBC = createDBC(self.project.db, database)
        projectDBC.dbc.open()

        self.variableTypeDictionary = {}
        query = QSqlQuery(projectDBC.dbc)
        query.exec_("""desc %s""" %tablename)

        FIELD, TYPE, NULL, KEY, DEFAULT, EXTRA = range(6)

        while query.next():
            field = query.value(FIELD).toString()
            type = query.value(TYPE).toString()
            null = query.value(NULL).toString()
            key = query.value(KEY).toString()
            default = query.value(DEFAULT).toString()
            extra = query.value(EXTRA).toString()

            self.variableTypeDictionary['%s' %field] = type
        projectDBC.dbc.close()

    def editProject(self):

        editWidget = QWizard()
        editWidget.setWindowTitle("Edit Project")
        editWidget.setWindowIcon(QIcon("./images/editproject.png"))
        editWidget.setWizardStyle(QWizard.ClassicStyle)
        editWidget.setOption(QWizard.NoBackButtonOnStartPage)

        self.page = SummaryPage()
        self.page.setTitle("Summary")
        self.page.projectLocationDummy = True
        self.page.projectDatabaseDummy = True
        self.page.fillPage(self.project)
        self.page.enableEditableWidgets()
        editWidget.addPage(self.page)


        if editWidget.exec_():
            check1 = (self.page.projectDescLineEdit.text() == self.project.description)
            check2 = (self.page.projectResolutionComboBox.currentText() == self.project.resolution)

            #print check1, check2, 'checkkkkkkkkksssssssssssssssssss'

            if check1 and check2:
                pass
            else:
                #print 'Poject properties altered'
                self.page.updateProject()
                self.project = self.page.project
                self.project.save()
                self.populate()

            if not check2 and not self.project.controlUserProv.userProv:

                #print 'Project resolution changed'
                autoImportSFDataInstance = AutoImportSF2000Data(self.project)
                autoImportSFDataInstance.createMasterSubSFTable()
                autoImportSFDataInstance.projectDBC.dbc.close()
                resolution = ('%s'%self.page.projectResolutionComboBox.currentText()).lower()
                tablename = 'mastersftable%s' %(resolution)
                self.project.save()
                self.populate()




    def importData(self):
        #QMessageBox.information(None, "Check", "Import Data", QMessageBox.Ok)
        dataprocesscheck = DataDialog(self.project)
        dataprocesscheck.exec_()
        self.populate()


    def populate(self):
        self.setEnabled(True)
        self.clear()

        projectAncestor = QTreeWidgetItem(self, [QString("Project: " + self.project.name)])
        informationParent = QTreeWidgetItem(projectAncestor, [QString("Information")])

        dummy = ""
        if self.project.region is not None:
            for i in self.project.region.keys():
                dummy = dummy + i + ", "+ self.project.region[i]+ "; "


        resolutionText = self.project.resolution

        if resolutionText == "Tract":
            resolution = 'Census Tract'
        elif resolutionText == "Blockgroup":
            resolution = 'Census Blockgroup'
        elif resolutionText == 'TAZ':
            resolution = 'Traffic Analysis Zone (TAZ)'
        else:
            resolution = 'County'



        informationItems = {"Location":self.project.location,
                            "Description":self.project.description,
                            "Region":dummy,
                            "Resolution":resolution}
        for i,j in informationItems.items():
            child = QTreeWidgetItem(informationParent, [i, QString(j)])

        geocorrParent = QTreeWidgetItem(projectAncestor, [QString("Geographic Correspondence")])
        geocorrUserProvText = self.userProvText(self.project.geocorrUserProv.userProv)
        geocorrItems = {"User Provided":geocorrUserProvText,
                        "Location":self.project.geocorrUserProv.location}

        for i,j in geocorrItems.items():
            child = QTreeWidgetItem(geocorrParent, [i, QString("%s"%j)])

        sampleParent = QTreeWidgetItem(projectAncestor, [QString("Sample")])
        sampleUserProvText = self.userProvText(self.project.sampleUserProv.userProv, 
                                               self.project.sampleUserProv.defSource)
        sampleItems = {"User Provided":sampleUserProvText,
                       "Household Data Location": self.project.sampleUserProv.hhLocation,
                       "GQ Data Location": self.project.sampleUserProv.gqLocation,
                       "Person Data Location": self.project.sampleUserProv.personLocation}

        for i,j in sampleItems.items():
            child = QTreeWidgetItem(sampleParent, [i, QString("%s"%j)])

        controlParent = QTreeWidgetItem(projectAncestor, [QString("Control")])
        controlUserProvText = self.userProvText(self.project.controlUserProv.userProv,
                                                self.project.controlUserProv.defSource)
        controlItems = {"User Provided":controlUserProvText,
                       "Household Data Location": self.project.controlUserProv.hhLocation,
                       "GQ Data Location": self.project.controlUserProv.gqLocation,
                       "Person Data Location": self.project.controlUserProv.personLocation}

        for i,j in controlItems.items():
            child = QTreeWidgetItem(controlParent, [i, QString("%s"%j)])

        dbParent = QTreeWidgetItem(projectAncestor, [QString("Database")])
        dbItems = {"Hostname":self.project.db.hostname,
                   "Username":self.project.db.username}
        #"Password":self.project.db.password}

        for i,j in dbItems.items():
            child = QTreeWidgetItem(dbParent, [QString(i), QString(j)])


        projectTableParent = QTreeWidgetItem(projectAncestor, [QString("Project Tables")])
        scenarioTableParent = QTreeWidgetItem(projectAncestor, [QString("Scenario Tables")])

        self.tableChildren(projectTableParent, 0)
        self.tableChildren(scenarioTableParent, 1)

        self.expandItem(projectAncestor)
        self.expandSort(informationParent, 0)
        self.expandSort(geocorrParent, 0)
        self.expandSort(sampleParent, 0)
        self.expandSort(controlParent, 0)
        self.expandSort(dbParent, 0)
        self.expandSort(projectTableParent, 0)
        self.expandSort(scenarioTableParent, 0)


    def userProvText(self, text, source='default'):
        if text:
            return "Yes"
        else:
            return "No, %s data will be used" %source



    def tableChildren(self, parent, scenario=0):

        if scenario > 0:
            databaseName = self.project.name + 'scenario' + str(self.project.scenario)
        else:
            databaseName = self.project.name

        projectDBC = createDBC(self.project.db, databaseName)

        projectDBC.dbc.open()

        self.query = QSqlQuery(projectDBC.dbc)

        if not self.query.exec_("""show tables"""):
            raise FileError, self.query.lastError().text()

        tableItems = []

        while self.query.next():
            tableItems.append(self.query.value(0).toString())

        projectDBC.dbc.close()

        tableItems.sort()

        for i in tableItems:
            child = QTreeWidgetItem(parent, [QString(i)])

    def expandSort(self, item, index):
        self.expandItem(item)
        item.sortChildren(index, Qt.AscendingOrder)


